package com.io.yutian.pluginsettingtool.frame;

import java.awt.datatransfer.DataFlavor;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.*;
import javax.swing.event.*;
import com.io.yutian.pluginsettingtool.data.GiftData;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.yaml.snakeyaml.Yaml;

import java.awt.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;

public class MainFrame extends JFrame {

    private File file;
    private FileConfiguration config;
    private Map<String, GiftData> giftDatas = new HashMap<>();

    public MainFrame() {
        initComponents();
        init(false);
    }

    private void init(boolean visible) {
        label2.setVisible(visible);
        label3.setVisible(visible);
        label4.setVisible(visible);
        comboBox1.setVisible(visible);
        comboBox2.setVisible(visible);
        comboBox3.setVisible(visible);
        button1.setVisible(visible);
    }

    private void load(File file) {
        if (!file.getName().endsWith(".yml")) {
            JOptionPane.showMessageDialog(this, "非法的文件类型", "", JOptionPane.WARNING_MESSAGE);
            return;
        }
        this.file = file;
        config = YamlConfiguration.loadConfiguration(file);
        giftDatas = new HashMap<>();
        if (config.contains("GiftData") && config.isConfigurationSection("GiftData")) {
            ConfigurationSection section = config.getConfigurationSection("GiftData");
            for (String key : section.getKeys(false)) {
                String event = section.getString(key+".event");
                String win = section.getString(key+".win");
                String sound = section.getString(key+".sound");
                giftDatas.put(key, new GiftData(event, win, sound));
            }
        }
        DefaultListModel<String> listModel = new DefaultListModel<>();
        List<String> keyList = new ArrayList<>(giftDatas.keySet());
        for (int i = keyList.size() - 1; i >= 0; i--) {
            listModel.addElement(keyList.get(i));
        }
        list1.setModel(listModel);
        init(true);
    }

    private void save(File file) {
        ConfigurationSection section = (config.contains("GiftData") && config.isConfigurationSection("GiftData")) ? config.getConfigurationSection("GiftData") : config.createSection("GiftData");
        for (Map.Entry<String, GiftData> entry : giftDatas.entrySet()) {
            String key = entry.getKey();
            GiftData giftData = entry.getValue();
            section.set(key+".event", giftData.getEvent());
            section.set(key+".win", giftData.getWin());
            section.set(key+".sound", giftData.getSound());
        }
        try {
            config.save(file);
        } catch (IOException e) {
            e.printStackTrace();
        }
        JOptionPane.showMessageDialog(this, "保存成功", "", JOptionPane.INFORMATION_MESSAGE);
    }

    private void reloadGiftData() {
        String value = (String) list1.getSelectedValue();
        if (value == null) {
            return;
        }
        GiftData giftData = giftDatas.get(value);
        if (giftData == null) {
            return;
        }
        comboBox1.setSelectedItem(giftData.getEvent());
        comboBox3.setSelectedItem(giftData.getSound());
    }

    private void list1ValueChanged(ListSelectionEvent e) {
        reloadGiftData();
    }

    private void button1(ActionEvent e) {
        save(file);
    }

    private void comboBox1ItemStateChanged(ItemEvent e) {
        String value = (String) list1.getSelectedValue();
        if (value == null) {
            return;
        }
        GiftData giftData = giftDatas.get(value);
        if (giftData == null) {
            return;
        }
        String v = (String) e.getItem();
        giftData.setEvent(v);
        giftDatas.put(value, giftData);
    }

    private void comboBox2ItemStateChanged(ItemEvent e) {
        String value = (String) list1.getSelectedValue();
        if (value == null) {
            return;
        }
        GiftData giftData = giftDatas.get(value);
        if (giftData == null) {
            return;
        }
        String v = (String) e.getItem();
        giftData.setWin(v);
        giftDatas.put(value, giftData);
    }

    private void comboBox3ItemStateChanged(ItemEvent e) {
        String value = (String) list1.getSelectedValue();
        if (value == null) {
            return;
        }
        GiftData giftData = giftDatas.get(value);
        if (giftData == null) {
            return;
        }
        String v = (String) e.getItem();
        giftData.setSound(v);
        giftDatas.put(value, giftData);
    }

    private void initComponents() {
        fileChooser = new JFileChooser();
        fileChooser.setAcceptAllFileFilterUsed(false);
        fileChooser.setMultiSelectionEnabled(false);
        fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        fileChooser.addChoosableFileFilter(new FileFilter() {
            @Override
            public boolean accept(File f) {
                return true;
            }

            @Override
            public String getDescription() {
                return "*.yml";
            }
        });
        DropTarget dropTarget = new DropTarget(this, DnDConstants.ACTION_COPY_OR_MOVE, new DropTargetAdapter() {
            @Override
            public void drop(DropTargetDropEvent dtde) {
                try {
                    if (dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
                        dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
                        java.util.List<File> list = (List<File>) dtde.getTransferable().getTransferData(DataFlavor.javaFileListFlavor);
                        if (list.size() > 0) {
                            File file = list.get(0);
                            load(file);
                            dtde.dropComplete(true);
                        } else {
                            dtde.dropComplete(false);
                        }
                    } else {
                        dtde.rejectDrop();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
        // JFormDesigner - Component initialization - DO NOT MODIFY  //GEN-BEGIN:initComponents  @formatter:off
        panel1 = new JPanel();
        scrollPane1 = new JScrollPane();
        list1 = new JList();
        label1 = new JLabel();
        label2 = new JLabel();
        label3 = new JLabel();
        label4 = new JLabel();
        comboBox1 = new JComboBox<>();
        comboBox2 = new JComboBox<>();
        comboBox3 = new JComboBox<>();
        button1 = new JButton();

        //======== this ========
        setTitle("\u63d2\u4ef6\u914d\u7f6e\u5de5\u5177");
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        setVisible(true);
        setResizable(false);
        Container contentPane = getContentPane();
        contentPane.setLayout(null);

        //======== panel1 ========
        {
            panel1.setLayout(null);

            //======== scrollPane1 ========
            {

                //---- list1 ----
                list1.addListSelectionListener(e -> list1ValueChanged(e));
                scrollPane1.setViewportView(list1);
            }
            panel1.add(scrollPane1);
            scrollPane1.setBounds(35, 30, 180, 375);

            //---- label1 ----
            label1.setText("\u793c\u7269");
            label1.setFont(label1.getFont().deriveFont(label1.getFont().getSize() + 3f));
            panel1.add(label1);
            label1.setBounds(new Rectangle(new Point(35, 5), label1.getPreferredSize()));

            //---- label2 ----
            label2.setText("\u6548\u679c");
            label2.setFont(label2.getFont().deriveFont(label2.getFont().getSize() + 3f));
            panel1.add(label2);
            label2.setBounds(270, 30, label2.getPreferredSize().width, 21);

            //---- label3 ----
            label3.setText("\u80dc\u5229");
            label3.setFont(label3.getFont().deriveFont(label3.getFont().getSize() + 3f));
            panel1.add(label3);
            label3.setBounds(270, 90, 30, 21);

            //---- label4 ----
            label4.setText("\u58f0\u97f3");
            label4.setFont(label4.getFont().deriveFont(label4.getFont().getSize() + 3f));
            panel1.add(label4);
            label4.setBounds(270, 150, 30, 21);

            //---- comboBox1 ----
            comboBox1.setModel(new DefaultComboBoxModel<>(new String[] {
                "\u6548\u679c1",
                "\u6548\u679c2",
                "\u6548\u679c3"
            }));
            comboBox1.addItemListener(e -> comboBox1ItemStateChanged(e));
            panel1.add(comboBox1);
            comboBox1.setBounds(new Rectangle(new Point(305, 25), comboBox1.getPreferredSize()));

            //---- comboBox2 ----
            comboBox2.setModel(new DefaultComboBoxModel<>(new String[] {
                "\u6548\u679c1",
                "\u6548\u679c2",
                "\u6548\u679c3"
            }));
            comboBox2.addItemListener(e -> comboBox2ItemStateChanged(e));
            panel1.add(comboBox2);
            comboBox2.setBounds(305, 85, 75, 28);

            //---- comboBox3 ----
            comboBox3.setModel(new DefaultComboBoxModel<>(new String[] {
                "\u6548\u679c1",
                "\u6548\u679c2",
                "\u6548\u679c3"
            }));
            comboBox3.addItemListener(e -> comboBox3ItemStateChanged(e));
            panel1.add(comboBox3);
            comboBox3.setBounds(305, 145, 75, 28);

            //---- button1 ----
            button1.setText("\u4fdd\u5b58");
            button1.setFont(new Font("Noto Sans", Font.BOLD, 14));
            button1.addActionListener(e -> button1(e));
            panel1.add(button1);
            button1.setBounds(500, 365, 80, 37);

            {
                // compute preferred size
                Dimension preferredSize = new Dimension();
                for(int i = 0; i < panel1.getComponentCount(); i++) {
                    Rectangle bounds = panel1.getComponent(i).getBounds();
                    preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                    preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
                }
                Insets insets = panel1.getInsets();
                preferredSize.width += insets.right;
                preferredSize.height += insets.bottom;
                panel1.setMinimumSize(preferredSize);
                panel1.setPreferredSize(preferredSize);
            }
        }
        contentPane.add(panel1);
        panel1.setBounds(0, 0, 630, 420);

        {
            // compute preferred size
            Dimension preferredSize = new Dimension();
            for(int i = 0; i < contentPane.getComponentCount(); i++) {
                Rectangle bounds = contentPane.getComponent(i).getBounds();
                preferredSize.width = Math.max(bounds.x + bounds.width, preferredSize.width);
                preferredSize.height = Math.max(bounds.y + bounds.height, preferredSize.height);
            }
            Insets insets = contentPane.getInsets();
            preferredSize.width += insets.right;
            preferredSize.height += insets.bottom;
            contentPane.setMinimumSize(preferredSize);
            contentPane.setPreferredSize(preferredSize);
        }
        pack();
        setLocationRelativeTo(getOwner());
        // JFormDesigner - End of component initialization  //GEN-END:initComponents  @formatter:on
    }

    private JFileChooser fileChooser;
    // JFormDesigner - Variables declaration - DO NOT MODIFY  //GEN-BEGIN:variables  @formatter:off
    private JPanel panel1;
    private JScrollPane scrollPane1;
    private JList list1;
    private JLabel label1;
    private JLabel label2;
    private JLabel label3;
    private JLabel label4;
    private JComboBox<String> comboBox1;
    private JComboBox<String> comboBox2;
    private JComboBox<String> comboBox3;
    private JButton button1;
    // JFormDesigner - End of variables declaration  //GEN-END:variables  @formatter:on
}
